#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'FirePass (F5 Networks)'


def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    return False


def check_schema_01(self):
    if not self.matchCookie('^VHOST'):
        return False

    if not self.matchHeader(('Location', r'\/my\.logon\.php3')):
        return False

    return True


def check_schema_02(self):
    if not self.matchCookie(r'^F5_fire.+?'):
        return False

    if not self.matchCookie('^F5_passid_shrinked'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Chuang Yu Shield (Yunaq)'


def is_waf(self):
    if self.matchContent(r'www\.365cyd\.com'):
        return True

    if self.matchContent(r'help\.365cyd\.com/cyd\-error\-help.html\?code=403'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Airlock (Phion/Ergon)'


def is_waf(self):
    # This method of detection is old (though most reliable), so we check it first
    if self.matchCookie(r'^al[_-]?(sess|lb)='):
        return True

    if self.matchContent(r'server detected a syntax error in your request'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'PerimeterX (PerimeterX)'


def is_waf(self):
    if self.matchContent(r'www\.perimeterx\.(com|net)/whywasiblocked'):
        return True

    if self.matchContent(r'client\.perimeterx\.(net|com)'):
        return True

    if self.matchContent(r'denied because we believe you are using automation tools'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'WebSEAL (IBM)'


def is_waf(self):
    if self.matchHeader(('Server', 'WebSEAL')):
        return True

    if self.matchContent(r"This is a WebSEAL error message template file"):
        return True

    if self.matchContent(r"WebSEAL server received an invalid HTTP request"):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'ThreatX (A10 Networks)'


def is_waf(self):
    if self.matchHeader(('X-Request-Id', r'.*')) and \
       self.matchContent(r"^Forbidden - ID: ([a-fA-F0-9]{32})$") and \
       self.matchStatus(403):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'DOSarrest (DOSarrest Internet Security)'


def is_waf(self):
    if self.matchHeader(('X-DIS-Request-ID', '.+')):
        return True

    # Found samples of DOSArrest returning 'Server: DoSArrest/3.5'
    if self.matchHeader(('Server', r'DOSarrest(.*)?')):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Kemp LoadMaster (Progress Software)'


def is_waf(self):
    if self.matchHeader(('X-ServedBy', 'KEMP-LM')) and \
        self.matchStatus(403) and \
        self.matchContent(r'<title>403 Forbidden</title>'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'DotDefender (Applicure Technologies)'


def is_waf(self):
    if self.matchHeader(('X-dotDefender-denied', r'.+?'), attack=True):
        return True

    if self.matchContent(r'dotdefender blocked your request'):
        return True

    if self.matchContent(r'Applicure is the leading provider of web application security'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Bekchy (Faydata Technologies Inc.)'


def is_waf(self):
    # Both signatures are contained within response, so checking for any one of them
    # Sometimes I observed that there is an XHR request being being made to submit the
    # report data automatically upon page load. In those cases a missing https is causing
    # false negatives.
    if self.matchContent(r'Bekchy.{0,10}?Access Denied'):
        return True

    if self.matchContent(r'bekchy\.com/report'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'ISA Server (Microsoft)'


def is_waf(self):
    if self.matchContent(r'The.{0,10}?(isa.)?server.{0,10}?denied the specified uniform resource locator \(url\)'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'AliYunDun (Alibaba Cloud Computing)'


def is_waf(self):
    if not self.matchContent(r'error(s)?\.aliyun(dun)?\.(com|net)?'):
        return False

    if not self.matchContent(r'alicdn\.com\/sd\-base\/static\/\d{1,2}\.\d{1,2}\.\d{1,2}\/image\/405\.png'):
        return False

    if not self.matchContent(r'Sorry, your request has been blocked as it may cause potential threats to the server\'s security.'):
        return False

    if not self.matchStatus(405):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Safe3 Web Firewall (Safe3)'


def is_waf(self):
    if self.matchHeader(('Server', 'Safe3 Web Firewall')):
        return True

    if self.matchHeader(('X-Powered-By', r'Safe3WAF/[\.0-9]+?')):
        return True

    if self.matchContent(r'Safe3waf/[0-9\.]+?'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

import time
import logging
from copy import copy

import requests
import urllib3

# For requests < 2.16, this should be used.
# requests.packages.urllib3.disable_warnings(InsecureRequestWarning)
# For requests >= 2.16, this is the convention
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

def_headers = {
    'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3',
    'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:130.0) Gecko/20100101 Firefox/130.0',
    'Accept-Language': 'en-US,en;q=0.5',
    'Upgrade-Insecure-Requests': '1',
    'Sec-Fetch-Dest': 'document',
    'Sec-Fetch-Mode': 'navigate',
    'Sec-Fetch-Site': 'cross-site',
    'Priority': 'u=0, i',
    'DNT': '1',
}
proxies = {}

class waftoolsengine:
    def __init__(
        self, target='https://example.com', debuglevel=0,
        path='/', proxies=None, redir=True, head=None, timeout=7
    ):
        self.target = target
        self.debuglevel = debuglevel
        self.requestnumber = 0
        self.path = path
        self.redirectno = 0
        self.allowredir = redir
        self.proxies = proxies
        self.log = logging.getLogger('wafw00f')
        self.timeout = timeout
        if head:
            self.headers = head
        else:
            self.headers = copy(def_headers) #copy object by value not reference. Fix issue #90

    def Request(self, headers=None, path=None, params={}, delay=0):
        try:
            time.sleep(delay)
            if not headers:
                h = self.headers
            else: h = headers
            req = requests.get(self.target, proxies=self.proxies, headers=h, timeout=self.timeout,
                    allow_redirects=self.allowredir, params=params, verify=False)
            self.log.info('Request Succeeded')
            self.log.debug('Headers: %s\n' % req.headers)
            self.log.debug('Content: %s\n' % req.content)
            self.requestnumber += 1
            return req
        except requests.exceptions.RequestException as e:
            self.log.error('Something went wrong %s' % (e.__str__()))



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Yunsuo (Yunsuo)'


def is_waf(self):
    if self.matchCookie(r'^yunsuo_session='):
        return True

    if self.matchContent(r'class=\"yunsuologo\"'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'AppWall (Radware)'


def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    return False


def check_schema_01(self):
    if self.matchContent(r'CloudWebSec\.radware\.com'):
        return True

    if self.matchHeader(('X-SL-CompState', '.+')):
        return True

    return False


def check_schema_02(self):
    if not self.matchContent(r'because we have detected unauthorized activity'):
        return False

    if not self.matchContent(r'<title>Unauthorized Request Blocked'):
        return False

    if not self.matchContent(r'if you believe that there has been some mistake'):
        return False

    if not self.matchContent(r'\?Subject=Security Page.{0,10}?Case Number'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'ZScaler (Accenture)'


def is_waf(self):
    if self.matchHeader(('Server', r'ZScaler')):
        return True

    if self.matchContent(r"Access Denied.{0,10}?Accenture Policy"):
        return True

    if self.matchContent(r'policies\.accenture\.com'):
        return True

    if self.matchContent(r'login\.zscloud\.net/img_logo_new1\.png'):
        return True

    if self.matchContent(r'Zscaler to protect you from internet threats'):
        return True

    if self.matchContent(r"Internet Security by ZScaler"):
        return True

    if self.matchContent(r"Accenture.{0,10}?webfilters indicate that the site likely contains"):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Armor Defense (Armor)'


def is_waf(self):
    if self.matchContent(r'blocked by website protection from armor'):
        return True

    if self.matchContent(r'please create an armor support ticket'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'OnMessage Shield (BlackBaud)'


def is_waf(self):
    if self.matchHeader(('X-Engine', 'onMessage Shield')):
        return True

    if self.matchContent(r'Blackbaud K\-12 conducts routine maintenance'):
        return True

    if self.matchContent(r'onMessage SHEILD'):
        return True

    if self.matchContent(r'maintenance\.blackbaud\.com'):
        return True

    if self.matchContent(r'status\.blackbaud\.com'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'CacheWall (Varnish)'


def is_waf(self):
    if self.matchHeader(('Server', 'Varnish')):
        return True

    if self.matchHeader(('X-Varnish', '.+')):
        return True

    if self.matchHeader(('X-Cachewall-Action', '.+?')):
        return True

    if self.matchHeader(('X-Cachewall-Reason', '.+?')):
        return True

    if self.matchContent(r'security by cachewall'):
        return True

    if self.matchContent(r'403 naughty.{0,10}?not nice!'):
        return True

    if self.matchContent(r'varnish cache server'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Viettel (Cloudrity)'


def is_waf(self):
    if self.matchContent(r"Access Denied.{0,10}?Viettel WAF"):
        return True

    if self.matchContent(r"cloudrity\.com\.(vn)?/"):
        return True

    if self.matchContent(r"Viettel WAF System"):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'WebTotem (WebTotem)'


def is_waf(self):
    if self.matchContent(r"The current request was blocked.{0,8}?>WebTotem"):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Squarespace (Squarespace)'


def is_waf(self):
    if self.matchHeader(('Server', 'Squarespace')):
        return True

    if self.matchCookie(r'^SS_ANALYTICS_ID='):
        return True

    if self.matchCookie(r'^SS_MATTR='):
        return True

    if self.matchCookie(r'^SS_MID='):
        return True

    if self.matchCookie(r'SS_CVT='):
        return True

    if self.matchContent(r'status\.squarespace\.com'):
        return True

    if self.matchContent(r'BRICK\-\d{2}'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Cloudbric (Penta Security)'


def is_waf(self):
    if self.matchContent(r'<title>Cloudbric.{0,5}?ERROR!'):
        return True

    if self.matchContent(r'Your request was blocked by Cloudbric'):
        return True

    if self.matchContent(r'please contact Cloudbric Support'):
        return True

    if self.matchContent(r'cloudbric\.zendesk\.com'):
        return True

    if self.matchContent(r'Cloudbric Help Center'):
        return True

    if self.matchContent(r'malformed request syntax.{0,4}?invalid request message framing.{0,4}?or deceptive request routing'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'CrawlProtect (Jean-Denis Brun)'


def is_waf(self):
    if self.matchCookie(r'^crawlprotecttag='):
        return True

    if self.matchContent(r'<title>crawlprotect'):
        return True

    if self.matchContent(r'this site is protected by crawlprotect'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'SiteGuard (EG Secure Solutions Inc.)'


def is_waf(self):
    if self.matchContent(r"Powered by SiteGuard"):
        return True

    if self.matchContent(r'The server refuse to browse the page'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'XLabs Security WAF (XLabs)'


def is_waf(self):
    if self.matchHeader(('X-CDN', r'XLabs Security')):
        return True

    if self.matchHeader(('Secured', r'^By XLabs Security')):
        return True

    if self.matchHeader(('Server', r'XLabs[-_]?.?WAF'), attack=True):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Sucuri CloudProxy (Sucuri Inc.)'


def is_waf(self):
    if self.matchHeader(('X-Sucuri-ID', r'.+?')):
        return True

    if self.matchHeader(('X-Sucuri-Cache', r'.+?')):
        return True

    if self.matchHeader(('Server', r'Sucuri(\-Cloudproxy)?')):
        return True

    if self.matchHeader(('X-Sucuri-Block', r'.+?'), attack=True):
        return True

    if self.matchContent(r"Access Denied.{0,6}?Sucuri Website Firewall"):
        return True

    if self.matchContent(r"<title>Sucuri WebSite Firewall.{0,6}?(CloudProxy)?.{0,6}?Access Denied"):
        return True

    if self.matchContent(r"sucuri\.net/privacy\-policy"):
        return True

    if self.matchContent(r"cdn\.sucuri\.net/sucuri[-_]firewall[-_]block\.css"):
        return True

    if self.matchContent(r'cloudproxy@sucuri\.net'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = '360PanYun (360 Technologies)'


def is_waf(self):
    if self.matchHeader(('Server', r'panyun')):
        return True
    if self.matchHeader(('X-Panyun-Request-ID', r'.+?'), attack=True):
        return True
    if self.matchHeader(('X-Panyun-Error-Reason', r'.+?'), attack=True):
        return True
    if self.matchHeader(('X-Panyun-Error-Step', r'.+?'), attack=True):
        return True
    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'eEye SecureIIS (BeyondTrust)'


def is_waf(self):
    if self.matchContent(r'SecureIIS is an internet security application'):
        return True

    if self.matchContent(r'Download SecureIIS Personal Edition'):
        return True

    if self.matchContent(r'https?://www\.eeye\.com/Secure\-?IIS'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'BIG-IP Local Traffic Manager (F5 Networks)'


def is_waf(self):
    if self.matchCookie('^bigipserver'):
        return True

    if self.matchHeader(('X-Cnection', 'close'), attack=True):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'ModSecurity (SpiderLabs)'


def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    if check_schema_03(self):
        return True

    return False


def check_schema_01(self):
    if self.matchHeader(('Server', r'(mod_security|Mod_Security|NOYB)')):
        return True

    if self.matchContent(r'This error was generated by Mod.?Security'):
        return True

    if self.matchContent(r'rules of the mod.security.module'):
        return True

    if self.matchContent(r'mod.security.rules triggered'):
        return True

    if self.matchContent(r'Protected by Mod.?Security'):
        return True

    if self.matchContent(r'/modsecurity[\-_]errorpage/'):
        return True

    if self.matchContent(r'modsecurity iis'):
        return True

    return False


def check_schema_02(self):
    if not self.matchReason('ModSecurity Action'):
        return False

    if not self.matchStatus(403):
        return False

    return True


def check_schema_03(self):
    if not self.matchReason('ModSecurity Action'):
        return False

    if not self.matchStatus(406):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'WP Cerber Security (Cerber Tech)'


def is_waf(self):
    if not self.matchContent(r'your request looks suspicious or similar to automated'):
        return False

    if not self.matchContent(r'our server stopped processing your request'):
        return False

    if not self.matchContent(r'We.re sorry.{0,10}?you are not allowed to proceed'):
        return False

    if not self.matchContent(r'requests from spam posting software'):
        return False

    if not self.matchContent(r'<title>403 Access Forbidden'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'SecureSphere (Imperva Inc.)'


def is_waf(self):
    if not self.matchContent(r'<(title|h2)>Error'):
        return False

    if not self.matchContent(r'The incident ID is'):
        return False

    if not self.matchContent(r"This page can't be displayed"):
        return False

    if not self.matchContent(r'Contact support for additional information'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'SiteGround (SiteGround)'


def is_waf(self):
    if self.matchContent(r"Our system thinks you might be a robot!"):
        return True

    if self.matchContent(r'access is restricted due to a security rule'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'AnYu (AnYu Technologies)'


def is_waf(self):
    if self.matchContent(r'anyu.{0,10}?the green channel'):
        return True

    if self.matchContent(r'your access has been intercepted by anyu'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Qiniu (Qiniu CDN)'


def is_waf(self):
    if self.matchHeader(('X-Qiniu-CDN', r'\d+?')):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'FortiWeb (Fortinet)'


def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    return False


def check_schema_01(self):
    if self.matchCookie(r'^FORTIWAFSID='):
        return True

    if self.matchContent('.fgd_icon'):
        return True

    return False


def check_schema_02(self):
    if not self.matchContent('fgd_icon'):
        return False

    if not self.matchContent('web.page.blocked'):
        return False

    if not self.matchContent('url'):
        return False

    if not self.matchContent('attack.id'):
        return False

    if not self.matchContent('message.id'):
        return False

    if not self.matchContent('client.ip'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2025, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Link11 WAAP (Link11)'

def is_waf(self):
    if check_schema_01(self):
        return True

    return False


def check_schema_01(self):
    if self.matchHeader(('server', 'rhino-core-shield')):
        return True

    return False
  



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'ServerDefender VP (Port80 Software)'


def is_waf(self):
    if self.matchHeader(('X-Pint', r'p(ort\-)?80')):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'NetScaler AppFirewall (Citrix Systems)'


def is_waf(self):
    # This header can be obtained without attack mode
    if self.matchHeader(('Via', r'NS\-CACHE')):
        return True

    # Cookies are set only when someone is authenticated.
    # Not much reliable since wafw00f isn't authenticating.
    if self.matchCookie(r'^(ns_af=|citrix_ns_id|NSC_)'):
        return True

    if self.matchContent(r'(NS Transaction|AppFW Session) id'):
        return True

    if self.matchContent(r'Violation Category.{0,5}?APPFW_'):
        return True

    if self.matchContent(r'Citrix\|NetScaler'):
        return True

    # Reliable but not all servers return this header
    if self.matchHeader(('Cneonction', r'^(keep alive|close)'), attack=True):
        return True

    if self.matchHeader(('nnCoection', r'^(keep alive|close)'), attack=True):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Sabre Firewall (Sabre)'


def is_waf(self):
    if self.matchContent(r'dxsupport\.sabre\.com'):
        return True

    if check_schema_01(self):
        return True

    return False


def check_schema_01(self):
    if not self.matchContent(r'<title>Application Firewall Error'):
        return False

    if not self.matchContent(r'add some important details to the email for us to investigate'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Oracle Cloud (Oracle)'


def is_waf(self):
    if self.matchContent(r'<title>fw_error_www'):
        return True

    if self.matchContent(r'src=\"/oralogo_small\.gif\"'):
        return True

    if self.matchContent(r'www\.oracleimg\.com/us/assets/metrics/ora_ocom\.js'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'DenyALL (Rohde & Schwarz CyberSecurity)'


def is_waf(self):
    if not self.matchStatus(200):
        return False

    if not self.matchReason('Condition Intercepted'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

# NOTE: this priority list is used so that each check can be prioritized,
# so that the quick checks are done first and ones that require more
# requests, are done later


wafdetectionsprio = [
	'360PanYun (360 Technologies)',
	'360WangZhanBao (360 Technologies)',
	'ACE XML Gateway (Cisco)',
	'ASP.NET Generic (Microsoft)',
	'ASPA Firewall (ASPA Engineering Co.)',
	'AWS Elastic Load Balancer (Amazon)',
	'AireeCDN (Airee)',
	'Airlock (Phion/Ergon)',
	'Alert Logic (Alert Logic)',
	'AliYunDun (Alibaba Cloud Computing)',
	'AnYu (AnYu Technologies)',
	'Anquanbao (Anquanbao)',
	'AppWall (Radware)',
	'Approach (Approach)',
	'Armor Defense (Armor)',
	'ArvanCloud (ArvanCloud)',
	'Astra (Czar Securities)',
	'Azion Edge Firewall (Azion)',
	'Azure Application Gateway (Microsoft)',
	'Azure Front Door (Microsoft)',
    'Baffin Bay (Mastercard)',
	'BIG-IP AP Manager (F5 Networks)',
	'BIG-IP AppSec Manager (F5 Networks)',
	'BIG-IP Local Traffic Manager (F5 Networks)',
	'Barikode (Ethic Ninja)',
	'Barracuda (Barracuda Networks)',
	'Bekchy (Faydata Technologies Inc.)',
	'Beluga CDN (Beluga)',
	'BinarySec (BinarySec)',
	'BitNinja (BitNinja)',
	'BlockDoS (BlockDoS)',
	'Bluedon (Bluedon IST)',
	'BulletProof Security Pro (AITpro Security)',
	'CacheFly CDN (CacheFly)',
	'CacheWall (Varnish)',
	'CdnNS Application Gateway (CdnNs/WdidcNet)',
	'ChinaCache Load Balancer (ChinaCache)',
	'Chuang Yu Shield (Yunaq)',
	'Cloud Protector (Rohde & Schwarz CyberSecurity)',
	'Cloudbric (Penta Security)',
	'Cloudflare (Cloudflare Inc.)',
	'Cloudfloor (Cloudfloor DNS)',
	'Cloudfront (Amazon)',
	'Comodo cWatch (Comodo CyberSecurity)',
	'CrawlProtect (Jean-Denis Brun)',
	'DDoS-GUARD (DDOS-GUARD CORP.)',
	'DOSarrest (DOSarrest Internet Security)',
	'DataPower (IBM)',
	'DenyALL (Rohde & Schwarz CyberSecurity)',
	'Distil (Distil Networks)',
	'DotDefender (Applicure Technologies)',
	'DynamicWeb Injection Check (DynamicWeb)',
	'Edgecast (Verizon Digital Media)',
	'Eisoo Cloud Firewall (Eisoo)',
	'Envoy (EnvoyProxy)',
	'Expression Engine (EllisLab)',
	'Fastly (Fastly CDN)',
	'FirePass (F5 Networks)',
	'FortiGate (Fortinet)',
	'FortiGuard (Fortinet)',
	'FortiWeb (Fortinet)',
	'GoDaddy Website Protection (GoDaddy)',
	'Google Cloud App Armor (Google Cloud)',
	'Greywizard (Grey Wizard)',
	'Huawei Cloud Firewall (Huawei)',
	'HyperGuard (Art of Defense)',
	'ISA Server (Microsoft)',
	'Imunify360 (CloudLinux)',
	'Incapsula (Imperva Inc.)',
	'IndusGuard (Indusface)',
	'Instart DX (Instart Logic)',
	'Janusec Application Gateway (Janusec)',
	'Jiasule (Jiasule)',
	'KS-WAF (KnownSec)',
	'Kemp LoadMaster (Progress Software)',
	'KeyCDN (KeyCDN)',
	'Kona SiteDefender (Akamai)',
	'LimeLight CDN (LimeLight)',
	'Link11 WAAP (Link11)',
	'LiteSpeed (LiteSpeed Technologies)',
	'Malcare (Inactiv)',
	'MaxCDN (MaxCDN)',
	'Mission Control Shield (Mission Control)',
	'ModSecurity (SpiderLabs)',
	'NAXSI (NBS Systems)',
	'NSFocus (NSFocus Global Inc.)',
	'Nemesida (PentestIt)',
	'NetContinuum (Barracuda Networks)',
	'NetScaler AppFirewall (Citrix Systems)',
	'NevisProxy (AdNovum)',
	'Newdefend (NewDefend)',
	'NexusGuard Firewall (NexusGuard)',
	'NinjaFirewall (NinTechNet)',
	'NullDDoS Protection (NullDDoS)',
	'OnMessage Shield (BlackBaud)',
	'Open-Resty Lua Nginx (FLOSS)',
	'Oracle Cloud (Oracle)',
	'PT Application Firewall (Positive Technologies)',
	'Palo Alto Next Gen Firewall (Palo Alto Networks)',
	'PentaWAF (Global Network Services)',
	'PerimeterX (PerimeterX)',
	'PowerCDN (PowerCDN)',
	'Profense (ArmorLogic)',
	'Puhui (Puhui)',
	'Qcloud (Tencent Cloud)',
	'Qiniu (Qiniu CDN)',
	'Qrator (Qrator)',
	'RSFirewall (RSJoomla!)',
	'RayWAF (WebRay Solutions)',
	'Reblaze (Reblaze)',
	'RequestValidationMode (Microsoft)',
	'SEnginx (Neusoft)',
	'Sabre Firewall (Sabre)',
	'Safe3 Web Firewall (Safe3)',
	'Safedog (SafeDog)',
	'Safeline (Chaitin Tech.)',
	'SecKing (SecKing)',
	'SecuPress WP Security (SecuPress)',
	'Secure Entry (United Security Providers)',
	'SecureSphere (Imperva Inc.)',
	'ServerDefender VP (Port80 Software)',
	'Shadow Daemon (Zecure)',
	'Shield Security (One Dollar Plugin)',
	'SiteGround (SiteGround)',
	'SiteGuard (EG Secure Solutions Inc.)',
	'Sitelock (TrueShield)',
	'SonicWall (Dell)',
	'Squarespace (Squarespace)',
	'SquidProxy IDS (SquidProxy)',
	'StackPath (StackPath)',
	'Sucuri CloudProxy (Sucuri Inc.)',
	'Tencent Cloud Firewall (Tencent Technologies)',
	'Teros (Citrix Systems)',
	'ThreatX (A10 Networks)',
	'Trafficshield (F5 Networks)',
	'TransIP Web Firewall (TransIP)',
	'UEWaf (UCloud)',
	'URLMaster SecurityCheck (iFinity/DotNetNuke)',
	'URLScan (Microsoft)',
	'UTM Web Protection (Sophos)',
	'Variti (Variti)',
	'Varnish (OWASP)',
	'Viettel (Cloudrity)',
	'VirusDie (VirusDie LLC)',
	'WP Cerber Security (Cerber Tech)',
	'WTS-WAF (WTS)',
	'Wallarm (Wallarm Inc.)',
	'WatchGuard (WatchGuard Technologies)',
	'WebARX (WebARX Security Solutions)',
	'WebKnight (AQTRONIX)',
	'WebLand (WebLand)',
	'WebSEAL (IBM)',
	'WebTotem (WebTotem)',
	'West263 CDN (West263CDN)',
	'Wordfence (Defiant)',
	'XLabs Security WAF (XLabs)',
	'Xuanwudun (Xuanwudun)',
	'YXLink (YxLink Technologies)',
	'Yundun (Yundun)',
	'Yunjiasu (Baidu Cloud Computing)',
	'Yunsuo (Yunsuo)',
	'ZScaler (Accenture)',
	'Zenedge (Zenedge)',
	'aeSecure (aeSecure)',
	'eEye SecureIIS (BeyondTrust)',
	'pkSecurity IDS (pkSec)',
	'wpmudev WAF (Incsub)',
]



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = '360WangZhanBao (360 Technologies)'


def is_waf(self):
    if self.matchHeader(('Server', r'qianxin\-waf')):
        return True

    if self.matchHeader(('WZWS-Ray', r'.+?')):
        return True

    if self.matchHeader(('X-Powered-By-360WZB', r'.+?')):
        return True

    if self.matchContent(r'wzws\-waf\-cgi/'):
        return True

    if self.matchContent(r'wangshan\.360\.cn'):
        return True

    if self.matchStatus(493):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'BulletProof Security Pro (AITpro Security)'


def is_waf(self):
    if not self.matchContent(r'\+?bpsMessage'):
        return False

    if not self.matchContent(r'403 Forbidden Error Page'):
        return False

    if not self.matchContent(r'If you arrived here due to a search'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Cloudfloor (Cloudfloor DNS)'


def is_waf(self):
    if self.matchHeader(('Server', r'CloudfloorDNS(.WAF)?')):
        return True

    if self.matchContent(r'<(title|h\d{1})>CloudfloorDNS.{0,6}?Web Application Firewall Error'):
        return True

    if self.matchContent(r'www\.cloudfloordns\.com/contact'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'AWS Elastic Load Balancer (Amazon)'


def is_waf(self):
    if self.matchHeader(('X-AMZ-ID', '.+?')):
        return True

    if self.matchHeader(('X-AMZ-Request-ID', '.+?')):
        return True

    if self.matchCookie(r'^aws.?alb='):
        return True

    if self.matchHeader(('Server', r'aws.?elb'), attack=True):
        return True

    if self.matchHeader(('X-Blocked-By-WAF', 'Blocked_by_custom_response_for_AWSManagedRules.*')):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'BIG-IP AppSec Manager (F5 Networks)'


def is_waf(self):
    if check_schema_01(self):
        return True

    # ASM ≥ 11.4.0 → eight hex digits after “TS” - https://my.f5.com/manage/s/article/K6850
    if self.matchCookie(r'TS[a-fA-F0-9]{8}=.+'):
        return True
    
    # ASM 10.0.0 – 11.3.0 → six hex digits after “TS” - https://my.f5.com/manage/s/article/K6850
    if self.matchCookie(r'TS[a-fA-F0-9]{6}=.+'):
        return True

    return False


def check_schema_01(self):
    if not self.matchContent('the requested url was rejected'):
        return False

    if not self.matchContent('please consult with your administrator'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Open-Resty Lua Nginx (FLOSS)'


def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    return False


def check_schema_01(self):
    if not self.matchHeader(('Server', r'^openresty/[0-9\.]+?')):
        return False

    if not self.matchStatus(403):
        return False

    return True


def check_schema_02(self):
    if not self.matchContent(r'openresty/[0-9\.]+?'):
        return False

    if not self.matchStatus(406):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Safedog (SafeDog)'


def is_waf(self):
    if self.matchCookie(r'^safedog\-flow\-item='):
        return True

    if self.matchHeader(('Server', 'Safedog')):
        return True

    if self.matchContent(r'safedogsite/broswer_logo\.jpg'):
        return True

    if self.matchContent(r'404\.safedog\.cn/sitedog_stat.html'):
        return True

    if self.matchContent(r'404\.safedog\.cn/images/safedogsite/head\.png'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'wpmudev WAF (Incsub)'


def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    return False


def check_schema_01(self):
    if not self.matchContent(r'href="http(s)?.\/\/wpmudev.com\/.{0,15}?'):
        return False

    if not self.matchContent(r'Click on the Logs tab, then the WAF Log.'):
        return False

    if not self.matchContent(r'Choose your site from the list'):
        return False

    if not self.matchStatus(403):
        return False

    return True


def check_schema_02(self):
    if not self.matchContent(r'<h1>Whoops, this request has been blocked!'):
        return False

    if not self.matchContent(r'This request has been deemed suspicious'):
        return False

    if not self.matchContent(r'possible attack on our servers.'):
        return False

    if not self.matchStatus(403):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Astra (Czar Securities)'


def is_waf(self):
    if self.matchCookie(r'^cz_astra_csrf_cookie'):
        return True

    if self.matchContent(r'astrawebsecurity\.freshdesk\.com'):
        return True

    if self.matchContent(r'www\.getastra\.com/assets/images'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Xuanwudun (Xuanwudun)'


def is_waf(self):
    if self.matchContent(r"admin\.dbappwaf\.cn/(index\.php/Admin/ClientMisinform/)?"):
        return True

    if self.matchContent(r'class=.(db[\-_]?)?waf(.)?([\-_]?row)?>'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Incapsula (Imperva Inc.)'


def is_waf(self):
    if self.matchCookie(r'^incap_ses.*?='):
        return True

    if self.matchCookie(r'^visid_incap.*?='):
        return True

    if self.matchContent(r'incapsula incident id'):
        return True

    if self.matchContent(r'powered by incapsula'):
        return True

    if self.matchContent(r'/_Incapsula_Resource'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'West263 CDN (West263CDN)'


def is_waf(self):
    if self.matchHeader(('X-Cache', r'WS?T263CDN')):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'NinjaFirewall (NinTechNet)'


def is_waf(self):
    if not self.matchContent(r'<title>NinjaFirewall.{0,10}?\d{3}.forbidden'):
        return False

    if not self.matchContent(r'For security reasons?.{0,10}?it was blocked and logged'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Wordfence (Defiant)'


def is_waf(self):
    if self.matchHeader(('Server', r'wf[_\-]?WAF')):
        return True

    if self.matchContent(r"Generated by Wordfence"):
        return True

    if self.matchContent(r'broke one of (the )?Wordfence (advanced )?blocking rules'):
        return True

    if self.matchContent(r"/plugins/wordfence"):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'YXLink (YxLink Technologies)'


def is_waf(self):
    if self.matchCookie(r'^yx_ci_session='):
        return True

    if self.matchCookie(r'^yx_language='):
        return True

    if self.matchHeader(('Server', r'Yxlink([\-_]?WAF)?')):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'URLScan (Microsoft)'


def is_waf(self):
    if self.matchContent(r"Rejected[-_]By[_-]UrlScan"):
        return True

    if self.matchContent(r'A custom filter or module.{0,4}?such as URLScan'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2021, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Shieldon Firewall (Shieldon.io)'


def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    if check_schema_03(self):
        return True

    if self.matchHeader((r'[Xx]-[Pp]rotected-[Bb]y', 'shieldon.io')):
        return True

    return False


def check_schema_01(self):
    if not self.matchContent('Please solve CAPTCHA'):
        return False

    if not self.matchContent('shieldon_captcha'):
        return False

    if not self.matchContent('Unusual behavior detected'):
        return False

    if not self.matchContent('status-user-info'):
        return False

    return True


def check_schema_02(self):
    if not self.matchContent('Access denied'):
        return False

    if not self.matchContent('The IP address you are using has been blocked.'):
        return False

    if not self.matchContent('status-user-info'):
        return False

    return True


def check_schema_03(self):
    if not self.matchContent('Please line up'):
        return False

    if not self.matchContent('This page is limiting the number of people online. Please wait a moment.'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Zenedge (Zenedge)'


def is_waf(self):
    if self.matchHeader(('Server', 'ZENEDGE')):
        return True

    if self.matchHeader(('X-Zen-Fury', r'.+?')):
        return True

    if self.matchContent(r'/__zenedge/'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'WebARX (WebARX Security Solutions)'


def is_waf(self):
    if self.matchContent(r"WebARX.{0,10}?Web Application Firewall"):
        return True

    if self.matchContent(r"www\.webarxsecurity\.com"):
        return True

    if self.matchContent(r'/wp\-content/plugins/webarx/includes/'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'BIG-IP AP Manager (F5 Networks)'


def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    if check_schema_03(self):
        return True

    return False


def check_schema_01(self):
    if not self.matchCookie('^LastMRH_Session'):
        return False

    if not self.matchCookie('^MRHSession'):
        return False

    return True


def check_schema_02(self):
    if not self.matchCookie('^MRHSession'):
        return False

    if not self.matchHeader(('Server', r'Big([-_])?IP'), attack=True):
        return False

    return True


def check_schema_03(self):
    if self.matchCookie('^F5_fullWT'):
        return True

    if self.matchCookie('^F5_fullWT'):
        return True

    if self.matchCookie('^F5_HT_shrinked'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'IndusGuard (Indusface)'


def is_waf(self):
    if self.matchHeader(('Server', r'IF_WAF')):
        return True

    if self.matchContent(r'This website is secured against online attacks. Your request was blocked'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2023, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'FortiGuard (Fortinet)'

def is_waf(self):
    if check_schema(self):
        return True

    return False

def check_schema(self):
    if not self.matchContent('FortiGuard Intrusion Prevention'):
        return False

    if not self.matchContent('//globalurl.fortinet.net'):
        return False

    if not self.matchContent('<title>Web Filter Violation'):
        return False
      
    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Cloudflare (Cloudflare Inc.)'


def is_waf(self):
    if self.matchHeader(('server', 'cloudflare')):
        return True

    if self.matchHeader(('server', r'cloudflare[-_]nginx')):
        return True

    if self.matchHeader(('cf-ray', r'.+?')):
        return True

    if self.matchCookie('__cfduid'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Cloudfront (Amazon)'


def is_waf(self):
    # This is standard detection schema, checking the server header
    if self.matchHeader(('Server', 'Cloudfront')):
        return True

    # Found samples returning 'Via: 1.1 58bfg7h6fg76h8fg7jhdf2.cloudfront.net (CloudFront)'
    if self.matchHeader(('Via', r'([0-9\.]+?)? \w+?\.cloudfront\.net \(Cloudfront\)')):
        return True

    # The request token is sent along with this header, eg:
    # X-Amz-Cf-Id: sX5QSkbAzSwd-xx3RbJmxYHL3iVNNyXa1UIebDNCshQbHxCjVcWDww==
    if self.matchHeader(('X-Amz-Cf-Id', '.+?'), attack=True):
        return True

    # This is another reliable fingerprint found on headers
    if self.matchHeader(('X-Cache', 'Error from Cloudfront'), attack=True):
        return True

    # These fingerprints are found on the blockpage itself
    if self.matchContent(r'Generated by cloudfront \(CloudFront\)'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'RequestValidationMode (Microsoft)'


def is_waf(self):
    if self.matchContent(r'Request Validation has detected a potentially dangerous client input'):
        return True

    if self.matchContent(r'ASP\.NET has detected data in the request'):
        return True

    if self.matchContent(r'HttpRequestValidationException'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'VirusDie (VirusDie LLC)'


def is_waf(self):
    if self.matchContent(r"cdn\.virusdie\.ru/splash/firewallstop\.png"):
        return True

    if self.matchContent(r'copy.{0,10}?Virusdie\.ru'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'URLMaster SecurityCheck (iFinity/DotNetNuke)'


def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    return False


def check_schema_01(self):
    if self.matchHeader(('X-UrlMaster-Debug', '.+')):
        return True

    if self.matchHeader(('X-UrlMaster-Ex', '.+')):
        return True

    return False


def check_schema_02(self):
    if not self.matchContent(r"Ur[li]RewriteModule"):
        return False

    if not self.matchContent(r'SecurityCheck'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'UTM Web Protection (Sophos)'


def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    return False


def check_schema_01(self):
    if self.matchContent(r'www\.sophos\.com'):
        return True

    if self.matchContent(r'Powered by.?(Sophos)? UTM Web Protection'):
        return True

    return False


def check_schema_02(self):
    if not self.matchContent(r'<title>Access to the requested URL was blocked'):
        return False

    if not self.matchContent(r'Access to the requested URL was blocked'):
        return False

    if not self.matchContent(r'incident was logged with the following log identifier'):
        return False

    if not self.matchContent(r'Inbound Anomaly Score exceeded'):
        return False

    if not self.matchContent(r'Your cache administrator is'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Nemesida (PentestIt)'


def is_waf(self):
    if self.matchContent(r'@?nemesida(\-security)?\.com'):
        return True

    if self.matchContent(r'Suspicious activity detected.{0,10}?Access to the site is blocked'):
        return True

    if self.matchContent(r'nwaf@'):
        return True

    if self.matchStatus(222):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Alert Logic (Alert Logic)'


def is_waf(self):
    if not self.matchContent(r'<(title|h\d{1})>requested url cannot be found'):
        return False

    if not self.matchContent(r'we are sorry.{0,10}?but the page you are looking for cannot be found'):
        return False

    if not self.matchContent(r'back to previous page'):
        return False

    if not self.matchContent(r'proceed to homepage'):
        return False

    if not self.matchContent(r'reference id'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'PentaWAF (Global Network Services)'


def is_waf(self):
    if self.matchHeader(('Server', r'PentaWaf(/[0-9\.]+)?')):
        return True

    if self.matchContent(r'Penta.?Waf/[0-9\.]+?.server'):
        return True

    return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

import csv
import io
import json
import logging
import os
import random
import re
import sys
import string
import urllib.parse
from collections import defaultdict
from optparse import OptionParser

from wafw00f import __license__, __version__
from wafw00f.lib.asciiarts import Color, randomArt
from wafw00f.lib.evillib import waftoolsengine
from wafw00f.manager import load_plugins
from wafw00f.wafprio import wafdetectionsprio


class WAFW00F(waftoolsengine):

    xsstring = r'<script>alert("XSS");</script>'
    sqlistring = r'UNION SELECT ALL FROM information_schema AND " or SLEEP(5) or "'
    lfistring = r'../../etc/passwd'
    rcestring = r'/bin/cat /etc/passwd; ping 127.0.0.1; curl google.com'
    xxestring = r'<!ENTITY xxe SYSTEM "file:///etc/shadow">]><pwn>&hack;</pwn>'

    def __init__(self, target='www.example.com', debuglevel=0, path='/',
                 followredirect=True, extraheaders={}, proxies=None, timeout=7):

        self.log = logging.getLogger('wafw00f')
        self.attackres = None
        waftoolsengine.__init__(self, target, debuglevel, path, proxies, followredirect, extraheaders, timeout)
        self.knowledge = {
            'generic': {
                'found': False,
                'reason': ''
            },
            'wafname': []
        }
        self.rq = self.normalRequest()

    def normalRequest(self):
        return self.Request()

    def customRequest(self, headers=None):
        return self.Request(
            headers=headers
        )

    def nonExistent(self):
        return self.Request(
            path=self.path + str(random.randrange(100, 999)) + '.html'
        )

    def xssAttack(self):
        return self.Request(
            path=self.path,
            params={
                create_random_param_name(): self.xsstring
            }
        )

    def xxeAttack(self):
        return self.Request(
            path=self.path,
            params={
                create_random_param_name(): self.xxestring
            }
        )

    def lfiAttack(self):
        return self.Request(
            path=self.path + self.lfistring
        )

    def centralAttack(self):
        return self.Request(
            path=self.path,
            params={
                create_random_param_name(): self.xsstring,
                create_random_param_name(): self.sqlistring,
                create_random_param_name(): self.lfistring
            }
        )

    def sqliAttack(self):
        return self.Request(
            path=self.path,
            params={
                create_random_param_name(): self.sqlistring
            }
        )

    def osciAttack(self):
        return self.Request(
            path=self.path,
            params= {
                create_random_param_name(): self.rcestring
            }
        )

    def performCheck(self, request_method):
        r = request_method()
        if r is None:
            raise RequestBlocked()
        return r, r.url

    # Most common attacks used to detect WAFs
    attcom = [xssAttack, sqliAttack, lfiAttack]
    attacks = [xssAttack, xxeAttack, lfiAttack, sqliAttack, osciAttack]

    def genericdetect(self):
        reason = ''
        reasons = ['Blocking is being done at connection/packet level.',
                   'The server header is different when an attack is detected.',
                   'The server returns a different response code when an attack string is used.',
                   'It closed the connection for a normal request.',
                   'The response was different when the request wasn\'t made from a browser.'
                ]
        try:
            # Testing for no user-agent response. Detects almost all WAFs out there.
            resp1, _ = self.performCheck(self.normalRequest)
            if 'User-Agent' in self.headers:
                self.headers.pop('User-Agent')  # Deleting the user-agent key from object not dict.
            resp3 = self.customRequest(headers=self.headers)
            if resp3 is not None and resp1 is not None:
                if resp1.status_code != resp3.status_code:
                    self.log.info('Server returned a different response when request didn\'t contain the User-Agent header.')
                    reason = reasons[4]
                    reason += '\r\n'
                    reason += 'Normal response code is "%s",' % resp1.status_code
                    reason += ' while the response code to a modified request is "%s"' % resp3.status_code
                    self.knowledge['generic']['reason'] = reason
                    self.knowledge['generic']['found'] = True
                    return True

            # Testing the status code upon sending a xss attack
            resp2, xss_url = self.performCheck(self.xssAttack)
            if resp1.status_code != resp2.status_code:
                self.log.info('Server returned a different response when a XSS attack vector was tried.')
                reason = reasons[2]
                reason += '\r\n'
                reason += 'Normal response code is "%s",' % resp1.status_code
                reason += ' while the response code to cross-site scripting attack is "%s"' % resp2.status_code
                self.knowledge['generic']['reason'] = reason
                self.knowledge['generic']['found'] = True
                return xss_url

            # Testing the status code upon sending a lfi attack
            resp2, lfi_url = self.performCheck(self.lfiAttack)
            if resp1.status_code != resp2.status_code:
                self.log.info('Server returned a different response when a directory traversal was attempted.')
                reason = reasons[2]
                reason += '\r\n'
                reason += 'Normal response code is "%s",' % resp1.status_code
                reason += ' while the response code to a file inclusion attack is "%s"' % resp2.status_code
                self.knowledge['generic']['reason'] = reason
                self.knowledge['generic']['found'] = True
                return lfi_url

            # Testing the status code upon sending a sqli attack
            resp2, sqli_url = self.performCheck(self.sqliAttack)
            if resp1.status_code != resp2.status_code:
                self.log.info('Server returned a different response when a SQLi was attempted.')
                reason = reasons[2]
                reason += '\r\n'
                reason += 'Normal response code is "%s",' % resp1.status_code
                reason += ' while the response code to a SQL injection attack is "%s"' % resp2.status_code
                self.knowledge['generic']['reason'] = reason
                self.knowledge['generic']['found'] = True
                return sqli_url

            # Checking for the Server header after sending malicious requests
            normalserver, attackresponse_server = '', ''
            response = self.attackres
            if 'server' in resp1.headers:
                normalserver = resp1.headers.get('Server')
            if response is not None and 'server' in response.headers:
                attackresponse_server = response.headers.get('Server')
            if attackresponse_server != normalserver:
                self.log.info('Server header changed, WAF possibly detected')
                self.log.debug('Attack response: %s' % attackresponse_server)
                self.log.debug('Normal response: %s' % normalserver)
                reason = reasons[1]
                reason += '\r\nThe server header for a normal response is "%s",' % normalserver
                reason += ' while the server header a response to an attack is "%s",' % attackresponse_server
                self.knowledge['generic']['reason'] = reason
                self.knowledge['generic']['found'] = True
                return True

        # If at all request doesn't go, press F
        except RequestBlocked:
            self.knowledge['generic']['reason'] = reasons[0]
            self.knowledge['generic']['found'] = True
            return True
        return False

    def matchHeader(self, headermatch, attack=False):
        if attack:
            r = self.attackres
        else:
            r = self.rq
        if r is None:
            return

        header, match = headermatch
        headerval = r.headers.get(header)
        if headerval:
            # set-cookie can have multiple headers, python gives it to us
            # concatinated with a comma
            if header == 'Set-Cookie':
                headervals = headerval.split(', ')
            else:
                headervals = [headerval]
            for headerval in headervals:
                if re.search(match, headerval, re.I):
                    return True
        return False

    def matchStatus(self, statuscode, attack=True):
        if attack:
            r = self.attackres
        else:
            r = self.rq
        if r is None:
            return
        if r.status_code == statuscode:
            return True
        return False

    def matchCookie(self, match, attack=False):
        return self.matchHeader(('Set-Cookie', match), attack=attack)

    def matchReason(self, reasoncode, attack=True):
        if attack:
            r = self.attackres
        else:
            r = self.rq
        if r is None:
            return
        # We may need to match multiline context in response body
        if str(r.reason) == reasoncode:
            return True
        return False

    def matchContent(self, regex, attack=True):
        if attack:
            r = self.attackres
        else:
            r = self.rq
        if r is None:
            return
        # We may need to match multiline context in response body
        if re.search(regex, r.text, re.I):
            return True
        return False

    wafdetections = dict()

    plugin_dict = load_plugins()
    result_dict = {}
    for plugin_module in plugin_dict.values():
        wafdetections[plugin_module.NAME] = plugin_module.is_waf
    # Check for prioritized ones first, then check those added externally
    checklist = wafdetectionsprio
    checklist += list(set(wafdetections.keys()) - set(checklist))

    def identwaf(self, findall=False):
        detected = list()
        try:
            self.attackres, xurl = self.performCheck(self.centralAttack)
        except RequestBlocked:
            return detected, None
        for wafvendor in self.checklist:
            self.log.info('Checking for %s' % wafvendor)
            if self.wafdetections[wafvendor](self):
                detected.append(wafvendor)
                if not findall:
                    break
        self.knowledge['wafname'] = detected
        return detected, xurl

def calclogginglevel(verbosity):
    default = 40  # errors are printed out
    level = default - (verbosity * 10)
    if level < 0:
        level = 0
    return level

def buildResultRecord(url, waf, evil_url=None):
    result = {}
    result['url'] = url
    if waf:
        result['detected'] = True
        if waf == 'generic':
            result['trigger_url'] = evil_url
            result['firewall'] = 'Generic'
            result['manufacturer'] = 'Unknown'
        else:
            result['trigger_url'] = evil_url
            result['firewall'] = waf.split('(')[0].strip()
            result['manufacturer'] = waf.split('(')[1].replace(')', '').strip()
    else:
        result['trigger_url'] = evil_url
        result['detected'] = False
        result['firewall'] = 'None'
        result['manufacturer'] = 'None'
    return result

def getTextResults(res=[]):
    # leaving out some space for future possibilities of newer columns
    # newer columns can be added to this tuple below
    keys = ('detected')
    res = [({key: ba[key] for key in ba if key not in keys}) for ba in res]
    rows = []
    for dk in res:
        p = [str(x) for _, x in dk.items()]
        rows.append(p)
    for m in rows:
        m[1] = '%s (%s)' % (m[1], m[2])
        m.pop()
    defgen = [
        (max([len(str(row[i])) for row in rows]) + 3)
        for i in range(len(rows[0]))
    ]
    rwfmt = ''.join(['{:>'+str(dank)+'}' for dank in defgen])
    textresults = []
    for row in rows:
        textresults.append(rwfmt.format(*row))
    return textresults

def create_random_param_name(size=8, chars=string.ascii_lowercase):
    return ''.join(random.choice(chars) for _ in range(size))

def disableStdOut():
    sys.stdout = None

def enableStdOut():
    sys.stdout = sys.__stdout__

def getheaders(fn):
    headers = {}
    if not os.path.exists(fn):
        logging.getLogger('wafw00f').critical('Headers file "%s" does not exist!' % fn)
        return
    with io.open(fn, 'r', encoding='utf-8') as f:
        for line in f.readlines():
            _t = line.split(':', 2)
            if len(_t) == 2:
                h, v = map(lambda x: x.strip(), _t)
                headers[h] = v
    return headers

class RequestBlocked(Exception):
    pass

def main():
    parser = OptionParser(usage='%prog url1 [url2 [url3 ... ]]\r\nexample: %prog http://www.victim.org/')
    parser.add_option('-v', '--verbose', action='count', dest='verbose', default=0,
                      help='Enable verbosity, multiple -v options increase verbosity')
    parser.add_option('-a', '--findall', action='store_true', dest='findall', default=False,
                      help='Find all WAFs which match the signatures, do not stop testing on the first one')
    parser.add_option('-r', '--noredirect', action='store_false', dest='followredirect',
                      default=True, help='Do not follow redirections given by 3xx responses')
    parser.add_option('-t', '--test', dest='test', help='Test for one specific WAF')
    parser.add_option('-o', '--output', dest='output', help='Write output to csv, json or text file depending on file extension. For stdout, specify - as filename.',
                      default=None)
    parser.add_option('-f', '--format', dest='format', help='Force output format to csv, json or text.',
                      default=None)
    parser.add_option('-i', '--input-file', dest='input', help='Read targets from a file. Input format can be csv, json or text. For csv and json, a `url` column name or element is required.',
                      default=None)
    parser.add_option('-l', '--list', dest='list', action='store_true',
                      default=False, help='List all WAFs that WAFW00F is able to detect')
    parser.add_option('-p', '--proxy', dest='proxy', default=None,
                      help='Use an HTTP proxy to perform requests, examples: http://hostname:8080, socks5://hostname:1080, http://user:pass@hostname:8080')
    parser.add_option('--version', '-V', dest='version', action='store_true',
                      default=False, help='Print out the current version of WafW00f and exit.')
    parser.add_option('--headers', '-H', dest='headers', action='store', default=None,
                      help='Pass custom headers via a text file to overwrite the default header set.')
    parser.add_option('-T', '--timeout', dest='timeout', action='store', default=7, type=int,
                      help='Set the timeout for the requests.')
    parser.add_option('--no-colors', dest='colors', action='store_false',
                      default=True, help='Disable ANSI colors in output.')

    options, args = parser.parse_args()

    logging.basicConfig(level=calclogginglevel(options.verbose))
    log = logging.getLogger('wafw00f')
    if options.output == '-':
        disableStdOut()

    # Windows based systems do not support ANSI sequences,
    # hence not displaying them.
    if not options.colors or 'win' in sys.platform:
        Color.disable()

    print(randomArt())
    (W,Y,G,R,B,C,E) = Color.unpack()

    if options.list:
        print('[+] Can test for these WAFs:\r\n')
        try:
            m = [i.replace(')', '').split(' (') for i in wafdetectionsprio]
            print(R+'  WAF Name'+' '*24+'Manufacturer\n  '+'-'*8+' '*24+'-'*12+'\n')
            max_len = max(len(str(x)) for k in m for x in k)
            for inner in m:
                first = True
                for elem in inner:
                    if first:
                        text = Y+'  {:<{}} '.format(elem, max_len+2)
                        first = False
                    else:
                        text = W+'{:<{}} '.format(elem, max_len+2)
                    print(text, E, end='')
                print()
            sys.exit(0)
        except Exception:
            return
    if options.version:
        print('[+] The version of WAFW00F you have is %sv%s%s' % (B, __version__, E))
        print('[+] WAFW00F is provided under the %s%s%s license.' % (C, __license__, E))
        return
    extraheaders = {}
    if options.headers:
        log.info('Getting extra headers from %s' % options.headers)
        extraheaders = getheaders(options.headers)
        if extraheaders is None:
            parser.error('Please provide a headers file with colon delimited header names and values')
    if len(args) == 0 and not options.input:
        parser.error('No test target specified.')
    #check if input file is present
    if options.input:
        log.debug('Loading file "%s"' % options.input)
        try:
            if options.input.endswith('.json'):
                with open(options.input) as f:
                    try:
                        urls = json.loads(f.read())
                    except json.decoder.JSONDecodeError:
                        log.critical('JSON file %s did not contain well-formed JSON', options.input)
                        sys.exit(1)
                log.info('Found: %s urls to check.' %(len(urls)))
                targets = [ item['url'] for item in urls ]
            elif options.input.endswith('.csv'):
                columns = defaultdict(list)
                with open(options.input) as f:
                    reader = csv.DictReader(f)
                    for row in reader:
                        for (k,v) in row.items():
                            columns[k].append(v)
                targets = columns['url']
            else:
                with open(options.input) as f:
                    targets = [x for x in f.read().splitlines()]
        except FileNotFoundError:
            log.error('File %s could not be read. No targets loaded.', options.input)
            sys.exit(1)
    else:
        targets = args
    results = []
    for target in targets:
        if not target.startswith('http'):
            log.info('The url %s should start with http:// or https:// .. fixing (might make this unusable)' % target)
            target = 'https://' + target
        print('[*] Checking %s' % target)
        pret = urllib.parse.urlparse(target)
        if pret is None:
            log.critical('The url %s is not well formed' % target)
            sys.exit(1)
        log.info('starting wafw00f on %s' % target)
        proxies = dict()
        if options.proxy:
            proxies = {
                'http': options.proxy,
                'https': options.proxy,
            }
        attacker = WAFW00F(target, debuglevel=options.verbose, path=pret.path,
                    followredirect=options.followredirect, extraheaders=extraheaders,
                        proxies=proxies, timeout=options.timeout)
        if attacker.rq is None:
            log.error('Site %s appears to be down' % pret.hostname)
            continue
        if options.test:
            if options.test in attacker.wafdetections:
                waf = attacker.wafdetections[options.test](attacker)
                if waf:
                    print('[+] The site %s%s%s is behind %s%s%s WAF.' % (B, target, E, C, options.test, E))
                else:
                    print('[-] WAF %s was not detected on %s' % (options.test, target))
            else:
                print('[-] WAF %s was not found in our list\r\nUse the --list option to see what is available' % options.test)
            return
        waf, xurl = attacker.identwaf(options.findall)
        log.info('Identified WAF: %s' % waf)
        if len(waf) > 0:
            for i in waf:
                results.append(buildResultRecord(target, i, xurl))
            print('[+] The site %s%s%s is behind %s%s%s WAF.' % (B, target, E, C, (E+' and/or '+C).join(waf), E))
        if (options.findall) or len(waf) == 0:
            print('[+] Generic Detection results:')
            generic_url = attacker.genericdetect()
            if generic_url:
                log.info('Generic Detection: %s' % attacker.knowledge['generic']['reason'])
                print('[*] The site %s seems to be behind a WAF or some sort of security solution' % target)
                print('[~] Reason: %s' % attacker.knowledge['generic']['reason'])
                results.append(buildResultRecord(target, 'generic', generic_url))
            else:
                print('[-] No WAF detected by the generic detection')
                results.append(buildResultRecord(target, None, None))
        print('[~] Number of requests: %s' % attacker.requestnumber)
    #print table of results
    if len(results) > 0:
        log.info('Found: %s matches.' % (len(results)))
    if options.output:
        if options.output == '-':
            enableStdOut()
            if options.format == 'json':
                json.dump(results, sys.stdout, indent=2, sort_keys=True)
            elif options.format == 'csv':
                csvwriter = csv.writer(sys.stdout, delimiter=',', quotechar='"',
                    quoting=csv.QUOTE_MINIMAL)
                count = 0
                for result in results:
                    if count == 0:
                        header = result.keys()
                        csvwriter.writerow(header)
                        count += 1
                    csvwriter.writerow(result.values())
            else:
                print(os.linesep.join(getTextResults(results)))
        elif options.output.endswith('.json'):
            log.debug('Exporting data in json format to file: %s' % (options.output))
            with open(options.output, 'w') as outfile:
                json.dump(results, outfile, indent=2, sort_keys=True)
        elif options.output.endswith('.csv'):
            log.debug('Exporting data in csv format to file: %s' % (options.output))
            with open(options.output, 'w') as outfile:
                csvwriter = csv.writer(outfile, delimiter=',', quotechar='"',
                    quoting=csv.QUOTE_MINIMAL)
                count = 0
                for result in results:
                    if count == 0:
                        header = result.keys()
                        csvwriter.writerow(header)
                        count += 1
                    csvwriter.writerow(result.values())
        else:
            log.debug('Exporting data in text format to file: %s' % (options.output))
            if options.format == 'json':
                with open(options.output, 'w') as outfile:
                    json.dump(results, outfile, indent=2, sort_keys=True)
            elif options.format == 'csv':
                with open(options.output, 'w') as outfile:
                    csvwriter = csv.writer(outfile, delimiter=',', quotechar='"',
                        quoting=csv.QUOTE_MINIMAL)
                    count = 0
                    for result in results:
                        if count == 0:
                            header = result.keys()
                            csvwriter.writerow(header)
                            count += 1
                        csvwriter.writerow(result.values())
            else:
                with open(options.output, 'w') as outfile:
                    outfile.write(os.linesep.join(getTextResults(results)))

if __name__ == '__main__':
    version_info = sys.version_info
    if version_info.major < 3 or (version_info.major == 3 and version_info.minor < 6):
        sys.stderr.write('Your version of python is way too old... please update to 3.6 or later\r\n')
    main()



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'WTS-WAF (WTS)'


def is_waf(self):
    if self.matchHeader(('Server', r'wts/[0-9\.]+?')):
        return True

    if self.matchContent(r"<(title|h\d{1})>WTS\-WAF"):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'LimeLight CDN (LimeLight)'


def is_waf(self):
    if self.matchCookie(r'^limelight'):
        return True

    if self.matchCookie(r'^l[mg]_sessid='):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'ASPA Firewall (ASPA Engineering Co.)'


def is_waf(self):
    if self.matchHeader(('Server', r'ASPA[\-_]?WAF')):
        return True

    if self.matchHeader(('ASPA-Cache-Status', r'.+?')):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Eisoo Cloud Firewall (Eisoo)'


def is_waf(self):
    if self.matchHeader(('Server', r'EisooWAF(\-AZURE)?/?')):
        return True

    if self.matchContent(r'<link.{0,10}?href=\"/eisoo\-firewall\-block\.css'):
        return True

    if self.matchContent(r'www\.eisoo\.com'):
        return True

    if self.matchContent(r'&copy; \d{4} Eisoo Inc'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Malcare (Inactiv)'


def is_waf(self):
    if self.matchContent(r'firewall.{0,15}?powered.by.{0,15}?malcare.{0,15}?pro'):
        return True

    if self.matchContent('blocked because of malicious activities'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'NAXSI (NBS Systems)'


def is_waf(self):
    if self.matchHeader(('X-Data-Origin', r'^naxsi(.+)?')):
        return True

    if self.matchHeader(('Server', r'naxsi(.+)?')):
        return True

    if self.matchContent(r'blocked by naxsi'):
        return True

    if self.matchContent(r'naxsi blocked information'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Varnish (OWASP)'


def is_waf(self):
    if self.matchContent(r'Request rejected by xVarnish\-WAF'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'ASP.NET Generic (Microsoft)'


def is_waf(self):
    if self.matchContent(r'iis (\d+.)+?detailed error'):
        return True

    if self.matchContent(r'potentially dangerous request querystring'):
        return True

    if self.matchContent(r'application error from being viewed remotely (for security reasons)?'):
        return True

    if self.matchContent(r'An application error occurred on the server'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'SquidProxy IDS (SquidProxy)'


def is_waf(self):
    if not self.matchHeader(('Server', r'squid(/[0-9\.]+)?')):
        return False

    if not self.matchContent(r'Access control configuration prevents your request'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Sitelock (TrueShield)'

# Well this is confusing, Sitelock itself uses Incapsula from Imperva
# So the fingerprints obtained on blockpage are similar to those of Incapsula.

def is_waf(self):
    if self.matchContent(r"SiteLock will remember you"):
        return True

    if self.matchContent(r"Sitelock is leader in Business Website Security Services"):
        return True

    if self.matchContent(r"sitelock[_\-]shield([_\-]logo|[\-_]badge)?"):
        return True

    if self.matchContent(r'SiteLock incident ID'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Shadow Daemon (Zecure)'


def is_waf(self):
    if not self.matchContent(r"<h\d{1}>\d{3}.forbidden<.h\d{1}>"):
        return False

    if not self.matchContent(r"request forbidden by administrative rules"):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Jiasule (Jiasule)'


def is_waf(self):
    if self.matchHeader(('Server', r'jiasule\-waf')):
        return True

    if self.matchCookie(r'^jsl_tracking(.+)?='):
        return True

    if self.matchCookie(r'__jsluid='):
        return True

    if self.matchContent(r'notice\-jiasule'):
        return True

    if self.matchContent(r'static\.jiasule\.com'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Huawei Cloud Firewall (Huawei)'


def is_waf(self):
    if self.matchCookie(r'^HWWAFSESID='):
        return True

    if self.matchHeader(('Server', r'HuaweiCloudWAF')):
        return True

    if self.matchContent(r'hwclouds\.com'):
        return True

    if self.matchContent(r'hws_security@'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Imunify360 (CloudLinux)'


def is_waf(self):
    if self.matchHeader(('Server', r'imunify360.{0,10}?')):
        return True

    if self.matchContent(r'protected.by.{0,10}?imunify360'):
        return True

    if self.matchContent(r'powered.by.{0,10}?imunify360'):
        return True

    if self.matchContent(r'imunify360.preloader'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Reblaze (Reblaze)'


def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    return False


def check_schema_01(self):
    if self.matchCookie(r'^rbzid'):
        return True

    if self.matchHeader(('Server', 'Reblaze Secure Web Gateway')):
        return True

    return False


def check_schema_02(self):
    if not self.matchContent(r'current session has been terminated'):
        return False

    if not self.matchContent(r'do not hesitate to contact us'):
        return False

    if not self.matchContent(r'access denied \(\d{3}\)'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'aeSecure (aeSecure)'


def is_waf(self):
    if self.matchHeader(('aeSecure-code', '.+?')):
        return True

    if self.matchContent(r'aesecure_denied\.png'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Cloud Protector (Rohde & Schwarz CyberSecurity)'

def is_waf(self):
    if self.matchContent(r'Cloud Protector.*?by Rohde.{3,8}?Schwarz Cybersecurity'):
        return True

    if self.matchContent(r"<a href='https?:\/\/(?:www\.)?cloudprotector\.com\/'>R.{1,6}?S.Cloud Protector"):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'PT Application Firewall (Positive Technologies)'


def is_waf(self):
    if not self.matchContent(r'<h1.{0,10}?Forbidden'):
        return False

    if not self.matchContent(r'<pre>Request.ID:.{0,10}?\d{4}\-(\d{2})+.{0,35}?pre>'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Yundun (Yundun)'


def is_waf(self):
    if self.matchHeader(('Server', 'YUNDUN')):
        return True

    if self.matchHeader(('X-Cache', 'YUNDUN')):
        return True

    if self.matchCookie(r'^yd_cookie='):
        return True

    if self.matchContent(r'Blocked by YUNDUN Cloud WAF'):
        return True

    if self.matchContent(r'yundun\.com/yd[-_]http[_-]error/'):
        return True

    if self.matchContent(r'www\.yundun\.com/(static/js/fingerprint\d{1}?\.js)?'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'StackPath (StackPath)'


def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    return False


def check_schema_01(self):
    if self.matchContent(r'<title>StackPath[^<]+</title>'):
        return True

    if self.matchContent(r'Protected by <a href="https?:\/\/(?:www\.)?stackpath\.com\/"[^>]+>StackPath'):
        return True

    return False


def check_schema_02(self):
    if not self.matchContent(r"is using a security service for protection against online attacks"):
        return False

    if not self.matchContent(r'An action has triggered the service and blocked your request'):
        return False

    return True



#!/usr/bin/env python
'''
Copyright (C) 2022, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Envoy (EnvoyProxy)'


def is_waf(self):
    if self.matchHeader(('server', 'envoy')):
        return True

    if self.matchHeader(('x-envoy-upstream-service-time', '.+')):
        return True

    if self.matchHeader(('x-envoy-downstream-service-cluster', '.+')):
        return True

    if self.matchHeader(('x-envoy-downstream-service-node', '.+')):
        return True

    if self.matchHeader(('x-envoy-external-address', '.+')):
        return True

    if self.matchHeader(('x-envoy-force-trace', '.+')):
        return True

    if self.matchHeader(('x-envoy-internal', '.+')):
        return True

    if self.matchHeader(('x-envoy-original-dst-host', '.+')):
        return True

    if self.matchHeader(('x-envoy-original-path', '.+')):
        return True

    if self.matchHeader(('x-envoy-local-overloaded', '.+')):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'NexusGuard Firewall (NexusGuard)'


def is_waf(self):
    if self.matchContent(r'Powered by Nexusguard'):
        return True

    if self.matchContent(r'nexusguard\.com/wafpage/.+#\d{3};'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'SonicWall (Dell)'


def is_waf(self):
    if self.matchHeader(('Server', 'SonicWALL')):
        return True

    if self.matchContent(r"<(title|h\d{1})>Web Site Blocked"):
        return True

    if self.matchContent(r'\+?nsa_banner'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Newdefend (NewDefend)'


def is_waf(self):
    # This header can be obtained without attack mode
    # Most reliable fingerprint
    if self.matchHeader(('Server', 'Newdefend')):
        return True

    # Reliable ones within blockpage
    if self.matchContent(r'www\.newdefend\.com/feedback'):
        return True

    if self.matchContent(r'/nd\-block/'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Greywizard (Grey Wizard)'


def is_waf(self):
    if self.matchHeader(('Server', 'greywizard')):
        return True

    if self.matchContent(r'<(title|h\d{1})>Grey Wizard'):
        return True

    if self.matchContent(r'contact the website owner or Grey Wizard'):
        return True

    if self.matchContent(r'We.ve detected attempted attack or non standard traffic from your ip address'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Yunjiasu (Baidu Cloud Computing)'


def is_waf(self):
    if self.matchHeader(('Server', r'yunjiasu.*')):
        return True

    if self.matchContent(r'href="/.well-known/yunjiasu-cgi/'):
        return True

    if self.matchContent(r"document.cookie='yjs_use_ob=0"):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Anquanbao (Anquanbao)'


def is_waf(self):
    if self.matchHeader(('X-Powered-By-Anquanbao', '.+?')):
        return True

    if self.matchContent(r'aqb_cc/error/'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'DDoS-GUARD (DDOS-GUARD CORP.)'


def is_waf(self):
    if self.matchCookie(r'^__ddg1.*?='):
        return True

    if self.matchCookie(r'^__ddg2.*?='):
        return True

    if self.matchCookie(r'^__ddgid.*?='):
        return True

    if self.matchCookie(r'^__ddgmark.*?='):
        return True

    if self.matchHeader(('Server', 'ddos-guard')):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Barracuda (Barracuda Networks)'


def is_waf(self):
    if self.matchCookie(r'^barra_counter_session='):
        return True

    if self.matchCookie(r'^BNI__BARRACUDA_LB_COOKIE='):
        return True

    if self.matchCookie(r'^BNI_persistence='):
        return True

    if self.matchCookie(r'^BN[IE]S_.*?='):
        return True

    if self.matchContent(r'Barracuda.Networks'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Bluedon (Bluedon IST)'


def is_waf(self):
    # Found sample servers returning 'Server: BDWAF/2.0'
    if self.matchHeader(('Server', r'BDWAF')):
        return True

    if self.matchContent(r'bluedon web application firewall'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'DynamicWeb Injection Check (DynamicWeb)'


def is_waf(self):
    if self.matchHeader(('X-403-Status-By', r'dw.inj.check'), attack=True):
        return True

    if self.matchContent(r'by dynamic check(.{0,10}?module)?'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'WebKnight (AQTRONIX)'


def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    if check_schema_03(self):
        return True

    return False


def check_schema_01(self):
    if not self.matchStatus(999):
        return False

    if not self.matchReason('No Hacking'):
        return False

    return True


def check_schema_02(self):
    if not self.matchStatus(404):
        return False

    if not self.matchReason('Hack Not Found'):
        return False

    return True


def check_schema_03(self):
    if self.matchContent(r'WebKnight Application Firewall Alert'):
        return True

    if self.matchContent(r'What is webknight\?'):
        return True

    if self.matchContent(r'AQTRONIX WebKnight is an application firewall'):
        return True

    if self.matchContent(r'WebKnight will take over and protect'):
        return True

    if self.matchContent(r'aqtronix\.com/WebKnight'):
        return True

    if self.matchContent(r'AQTRONIX.{0,10}?WebKnight'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Qcloud (Tencent Cloud)'


def is_waf(self):
    if not self.matchContent(r'腾讯云Web应用防火墙'):
        return False

    if not self.matchStatus(403):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2023, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'FortiGate (Fortinet)'

def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    return False

def check_schema_01(self):
    if not self.matchContent('//globalurl.fortinet.net'):
        return False

    if not self.matchContent('FortiGate Application Control'):
        return False

    return True

def check_schema_02(self):
    if not self.matchContent('Web Application Firewall'):
        return False

    if not self.matchContent('Event ID'):
        return False

    if not self.matchContent('//globalurl.fortinet.net'):
        return False

    return True



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Distil (Distil Networks)'


def is_waf(self):
    if self.matchContent(r'cdn\.distilnetworks\.com/images/anomaly\.detected\.png'):
        return True

    if self.matchContent(r'distilCaptchaForm'):
        return True

    if self.matchContent(r'distilCallbackGuard'):
        return True

    return False



#!/usr/bin/env python
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Azure Application Gateway (Microsoft)'


def is_waf(self):
    if self.matchContent(r'<center>Microsoft-Azure-Application-Gateway/v2</center>') and \
        self.matchContent(r'<h1>403 Forbidden</h1>'):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'Shield Security (One Dollar Plugin)'


def is_waf(self):
    if self.matchContent(r"You were blocked by the Shield"):
        return True

    if self.matchContent(r"remaining transgression\(s\) against this site"):
        return True

    if self.matchContent(r"Something in the URL.{0,5}?Form or Cookie data wasn\'t appropriate"):
        return True

    return False



#!/usr/bin/env python3
'''
Copyright (C) 2024, WAFW00F Developers.
See the LICENSE file for copying permission.
'''

NAME = 'pkSecurity IDS (pkSec)'


def is_waf(self):
    if check_schema_01(self):
        return True

    if check_schema_02(self):
        return True

    return False


def check_schema_01(self):
    if self.matchContent(r'pk.?Security.?Module'):
        return True

    if self.matchContent(r'Security.Alert'):
        return True

    return False


def check_schema_02(self):
    if not self.matchContent(r'As this could be a potential hack attack'):
        return False

    if not self.matchContent(r'A safety critical (call|request) was (detected|discovered) and blocked'):
        return False

    if not self.matchContent(r'maximum number of reloads per minute and prevented access'):
        return False

    return True
